/**
 * index.ts: logging utilities
 *
 * See:
 *  - https://github.com/gremo/nest-winston
 *  - https://github.com/winstonjs/winston
 *
 * (C) 2023
 * Price.com.hk
 */

import * as winston from 'winston';
import { utilities as nestWinstonModuleUtilities } from 'nest-winston';

const { combine, timestamp, printf, colorize } = winston.format;

//   Wrapper of createLogger, with the common options used for Price's usvc
export function createLogger(options?: winston.LoggerOptions): winston.Logger {
  const logLevel = process.env.LOG_LEVEL
    ? process.env.LOG_LEVEL
    : process.env.NODE_ENV === 'production'
    ? 'info'
    : 'debug';

  if (process.env.NODE_ENV === 'production') {
    return winston.createLogger({
      ...options,
      transports: [new winston.transports.Console()],
      format: combine(
        nestWinstonModuleUtilities.format.nestLike(),
        winston.format.json(),
        winston.format((info) => {
          // Remove reduntant 'value' while object is logged instead of msg string
          if (info.value) {
            delete info.value;
          }
          // Remove timestamp (for changing timestamp to 'time')
          if (info.timestamp) {
            delete info.timestamp;
            return info;
          }
          return info;
        })(),
        timestamp({
          // format: 'YYYY-MM-DDTHH:mm:ss.SSSZZ',
          format: 'YYYY-MM-DDTHH:mm:ssZZ',
          alias: 'time',
        }),
      ),
      level: logLevel,
    });
  } else {
    return winston.createLogger({
      ...options,
      transports: [new winston.transports.Console()],
      format: combine(
        nestWinstonModuleUtilities.format.nestLike(),
        // timestamp({ format: 'YYYY-MM-DD HH:mm:ss' }),
        timestamp({ format: 'HH:mm:ss' }),
        colorize(),
        // Format for debug-console, not in JSON meta param is ensured by splat()
        printf(
          ({
            context,
            namespace,
            module,
            timestamp,
            level,
            message,
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            value, // No print
            ...meta
          }) => {
            const infoContext = context ? `[${context}] ` : '';
            const infoString =
              module || namespace ? `[${module || namespace}] ` : '';
            return `${timestamp} ${level.padEnd(
              15,
            )} ${infoString}${infoContext}${message}${
              Object.keys(meta).length > 0 ? ' - ' + JSON.stringify(meta) : ''
            }`;
          },
        ),
      ),
      level: logLevel,
    });
  }
}
