import { Injectable, Logger, NotFoundException } from '@nestjs/common';
import { User } from './entities/user.entity';

@Injectable()
export class UserRepository {
  constructor(private readonly logger: Logger) {}

  private readonly users = new Map<number, User>();

  public async findAll(): Promise<User[]> {
    return [...this.users.values()];
  }

  public async get(id: number): Promise<User> {
    // const user = this.users.find((user) => user.id === id);
    if (!this.users.has(id)) {
      throw new NotFoundException('User not found');
    }
    return this.users.get(id);
  }

  public async add(user: User): Promise<void> {
    const largestId = Math.max(...Array.from(this.users.keys()));

    // Can get size from map directly but we simulate reading from external service
    // So calling the method instead
    // const cnt = await this.count();
    // await this.get(cnt - 1)
    //   .then((lastuser) => {
    //     id = lastuser.id + 1;
    //   })
    //   .catch(() => {
    //     id = 0;
    //   });

    user.id = largestId < 0 ? 0 : largestId + 1;
    this.users.set(user.id, user);
    this.logger.log({ message: 'User added', user });
  }

  public async update(id: number, user: User): Promise<User> {
    // Can get size from map directly but we simulate reading from external service
    // So calling the method instead
    // It will throw exception if not found.
    const oldUser = await this.get(id);

    this.users.set(id, user);
    this.logger.log({ message: 'User updated', from: oldUser, to: user });
    return oldUser;
  }

  public async delete(id: number): Promise<User> {
    let deletedUser: User;
    await this.get(id)
      .then((user) => {
        this.users.delete(id);
        this.logger.log({ message: 'User deleted', user });
        deletedUser = user;
      })
      .catch(() => {
        throw new NotFoundException(
          `User not found, No user deleted, id=${id}`,
        );
      });

    return deletedUser;
  }

  public async count(): Promise<number> {
    return this.users.size;
  }
}
